require "import"
import "android.content.Context"
import "android.os.Environment"
import "java.io.File"
import "android.widget.*"
import "com.androlua.*"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"
import "java.util.Locale"
import "android.content.Intent"

local context = service
local storageDir = Environment.getExternalStorageDirectory().toString() .. "/creador de documentos Mario Galván"

-- Crear carpeta si no existe
local fileDir = File(storageDir)
if not fileDir.exists() then
    fileDir.mkdirs()
end

-- Función para mostrar un mensaje accesible
function mostrarMensaje(mensaje)
    Toast.makeText(context, mensaje, Toast.LENGTH_LONG).show()
end

-- Función para procesar los resultados del reconocimiento de voz
function processResults(results, editText)
    local data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
    if data ~= nil and data.size() > 0 then
        local recognizedText = data.get(0)
        local currentText = editText.getText().toString()
        editText.setText(currentText .. " " .. recognizedText)
    end
    kill()
end

-- Función para destruir el reconocedor de voz
function kill()
    if speechRecognizer then
        speechRecognizer.destroy()
        speechRecognizer = nil
    end
end

-- Función para iniciar el reconocimiento de voz y colocar el texto en el cuadro de edición correspondiente
function startListening(editText)
    speechRecognizer = SpeechRecognizer.createSpeechRecognizer(context)
    local speechListener = RecognitionListener {
        onResults = function(results) processResults(results, editText) end,
        onError = function() mostrarMensaje("Error en el reconocimiento de voz") end
    }
    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, Locale.getDefault())
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, context.getPackageName())

    speechRecognizer.setRecognitionListener(speechListener)
    speechRecognizer.startListening(recognizerIntent)
end

-- Función para borrar el texto de un cuadro de edición
function borrarTexto(editText)
    editText.setText("")
end

-- Función para abrir la interfaz de creación de documentos
function abrirCreador()
    local layoutCreador = {
        LinearLayout,
        orientation = "vertical",
        padding = "16dp",
        {
            TextView,
            text = "Creador de Documentos, aquí ingresa los datos del documento.",
            textSize = "18sp",
            gravity = "center",
        },
        {
            LinearLayout,
            orientation = "horizontal",
            {
                EditText,
                id = "titulo",
                hint = "Ingresa el título del documento",
                layout_width = "0dp",
                layout_weight = "1",
            },
            {
                Button,
                text = "Dictado",
                onClick = function() startListening(titulo) end,
            },
            {
                Button,
                text = "Borrar",
                onClick = function() borrarTexto(titulo) end,
            },
        },
        {
            LinearLayout,
            orientation = "horizontal",
            {
                EditText,
                id = "contenido",
                hint = "Ingresa el contenido del documento",
                layout_width = "0dp",
                layout_height = "200dp",
                layout_weight = "1",
            },
            {
                Button,
                text = "Dictado",
                onClick = function() startListening(contenido) end,
            },
            {
                Button,
                text = "Borrar",
                onClick = function() borrarTexto(contenido) end,
            },
        },
        {
            LinearLayout,
            orientation = "horizontal",
            gravity = "center",
            {
                Button,
                text = "Cancelar",
                onClick = function() dlgCreador.dismiss() end,
            },
            {
                Button,
                text = "Aceptar",
                onClick = function()
                    local tituloDoc = titulo.getText().toString()
                    local contenidoDoc = contenido.getText().toString()
                    if tituloDoc ~= "" and contenidoDoc ~= "" then
                        local filePath = storageDir .. "/" .. tituloDoc .. ".txt"
                        local file = io.open(filePath, "w")
                        if file then
                            file:write(contenidoDoc)
                            file:close()
                            mostrarMensaje("Documento guardado")
                            service.asyncSpeak("Documento creado")
                            dlgCreador.dismiss()
                        else
                            mostrarMensaje("Error al guardar el documento")
                        end
                    else
                        mostrarMensaje("Completa todos los campos")
                        service.asyncSpeak("Primero ingresa los dos textos del documento")
                    end
                end,
            },
        },
    }

    dlgCreador = LuaDialog(service)
    dlgCreador.View = loadlayout(layoutCreador)
    dlgCreador.show()
end

-- Interfaz principal sin el botón "Ver tus documentos creados"
local layoutPrincipal = {
    LinearLayout,
    orientation = "vertical",
    padding = "16dp",
    {
        TextView,
        text = "Creador de Documentos",
        textSize = "22sp",
        gravity = "center",
    },
    {
        TextView,
        text = "Creado por Mario Galván",
        textSize = "16sp",
        gravity = "center",
        padding = "8dp",
    },
    {
        Button,
        text = "Crear nuevo documento",
        onClick = abrirCreador,
    },
    {
        Button,
        text = "Cerrar",
        onClick = function() dlgPrincipal.dismiss() end,
    },
}

dlgPrincipal = LuaDialog(service)
dlgPrincipal.View = loadlayout(layoutPrincipal)
dlgPrincipal.show()